#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.01.30 21:00:00                  #
# ================================================== #

__author__ = "Marcin Szczygliński"
__copyright__ = "Copyright 2024, Marcin Szczygliński"
__credits__ = ["Marcin Szczygliński"]
__license__ = "MIT"
__version__ = "2.0.133"
__build__ = "2024.01.30"
__maintainer__ = "Marcin Szczygliński"
__github__ = "https://github.com/szczyglis-dev/py-gpt"
__website__ = "https://pygpt.net"
__pypi__ = "https://pypi.org/project/pygpt-net"
__snap__ = "https://snapcraft.io/pygpt"
__documentation__ = "https://pygpt.readthedocs.io"
__email__ = "info@pygpt.net"
